/* -*-C-*- xstdlib.h */
/**********************************************************************/
/****************************** stdlib ********************************/
/**********************************************************************/

#ifndef STDLIB_H_DEFINED__
#define STDLIB_H_DEFINED__

/***********************************************************************
This file provides function prototype definitions for the C library.  It
can be used for environments where no system stdlib.h file is available,
such as for GNU gcc in non-ANSI UNIX C implementations.	 Note that some
older implementations may have different function return values, so some
care should be exercised in using this file.

Functions are ordered alphabetically below in two groups, each of two
sections, one with ANSI-style prototypes, and one with old-style
prototypes.

The first group contains the functions that belong to ANSI stdlib.h.

The second group contains miscellaneous other library functions that
have prototypes in other ANSI header files, but often lack prototypes in
typical non-ANSI C implementations.

[25-Nov-88]
***********************************************************************/

#if    STDC || HAVE_STDLIB_H
#if    IBM_PC_LATTICE
#include <s:/sys/lc/stdlib.h>		/* get the host version */
#else /* NOT IBM_PC_LATTICE */
#include <stdlib.h>		/* get the host version */
#ifdef sun				/* SunOS 4.1 stdlib.h is incomplete */
double		strtod ARGS((const char *nptr, char** endptr));
char		*strtok ARGS((char *s,const char *sepchars));
long		strtol ARGS((const char *nptr, char** endptr, int base));
unsigned long	strtoul ARGS((const char *nptr, char** endptr, int base));
#endif
#endif /* IBM_PC_LATTICE */

#ifdef _AIX
#include <unistd.h>	/* for EXIT_SUCCESS and EXIT_FAILURE */
			/* which SHOULD be in stdlib.h, but are not */
			/* on AIX mainframe and PS/2 implementations */
#endif /* _AIX */

#else  /* NOT (STDC || HAVE_STDLIB_H) */
/***********************************************************************
The real ANSI stdlib.h contents.  This	section is lacking definitions
of NULL, RAND_MAX, MB_CUR_MAX, which are usually system-dependent.  It
is   also   lacking   macros  and  functions	for the	 controversial
wide-character	type.  NULL  is generally available  in stdio.h, which
virtually all C programs include; it is also  available	 in the ANSI C
stddef.h.
***********************************************************************/

typedef int onexit_t;
typedef struct { int quot; int rem; } div_t;
typedef struct { long quot; long rem; } ldiv_t;

#ifdef ungetc
/* Turbo C makes this a macro which conflicts with prototypes below */
#undef ungetc
#endif /* ungetc */

void		abort ARGS((void));
int		abs ARGS((int j));
int		atexit ARGS((void (*func)(void)));
double		atof ARGS((const char *nptr));
int		atoi ARGS((const char *nptr));
long		atol ARGS((const char *nptr));
char		*bsearch ARGS((const void *key, const void *base,size_t nmemb,
			size_t size,int (*compar)(const void*, const void* )));
void		*calloc ARGS((size_t nmemb, size_t size));
div_t		div ARGS((int numer, int denom));
void		exit ARGS((int status));
void		free ARGS((void *ptr));
char		*getenv ARGS((const char *name));
long		labs ARGS((long j));
ldiv_t		ldiv ARGS((long numer, long denom));
void		*malloc ARGS((size_t size));
onexit_t	onexit ARGS((onexit_t (*func)(void)));
void		qsort ARGS((void *base, size_t nmemb, size_t size,
			int (*compar)(const void* , const void* )));
int		rand ARGS((void));
void		*realloc ARGS((void *ptr, size_t size));
void		srand ARGS((unsigned int seed));
double		strtod ARGS((const char *nptr, char** endptr));
char		*strtok ARGS((char *s,const char *sepchars));
long		strtol ARGS((const char *nptr, char** endptr, int base));
unsigned long	strtoul ARGS((const char *nptr, char** endptr, int base));
int		system ARGS((const char *string));

/***********************************************************************
The following group contains miscellaneous functions that are often
not prototyped in non-ANSI C implementations.
***********************************************************************/

int		fclose ARGS((FILE *file_ptr));
FILE		*fdopen ARGS((int file_desc, const char *a_mode));
int		fflush ARGS((FILE *file_ptr));
int		fgetc ARGS((FILE *file_ptr));
char		*fgetname ARGS((FILE *file_ptr, char *buffer, ...));
int		fgetpos ARGS((FILE *file_ptr, fpos_t *position));
char		*fgets ARGS((char *str, int maxchar, FILE *file_ptr));

#ifndef __GNUC__	/* complains of conflict with stdio.h definition */
#ifndef ardent
FILE		*fopen ARGS((const char *filespec, const char *a_mode, ...));
#endif /* ardent */
#endif /* __GNUC__ */

int		fprintf ARGS((FILE *file_ptr, const char *format_spec, ...));
int		fputc ARGS((int character, FILE *file_ptr));
int		fputs ARGS((const char *str, FILE *file_ptr));

#if    KCC_20
int		fread ARGS((void *ptr, size_t size_of_item,
			size_t number_items, FILE *file_ptr));
#else /* NOT KCC_20 */
#ifndef ardent
size_t		fread ARGS((void *ptr, size_t size_of_item,
			size_t number_items, FILE *file_ptr));
#endif /* ardent */
#endif /* KCC_20 */

#ifndef __GNUC__	/* complains of conflict with stdio.h definition */
#ifndef ardent
FILE		*freopen ARGS((const char *file_spec, const char *a_mode,
			FILE *file_ptr, ...));
#endif /* ardent */
#endif /* __GNUC__ */

int		fscanf ARGS((FILE *file_ptr, const char *format_spec, ...));
int		fseek ARGS((FILE *file_ptr, long offset, int direction));
int		fsetpos ARGS((FILE *file_ptr, const fpos_t *position));
long		ftell ARGS((FILE *file_ptr));

#if    KCC_20
int		fwrite ARGS((void *ptr, size_t size_of_item,
			     size_t number_items, FILE *file_ptr));
#else /* NOT KCC_20 */
#ifndef ardent
size_t		fwrite ARGS((void *ptr, size_t size_of_item,
			     size_t number_items, FILE *file_ptr));
#endif /* ardent */
#endif /* KCC_20 */

char		*gets ARGS((char *str));
int		getw ARGS((FILE *file_ptr));
int		printf ARGS((const char *format_spec, ...));
int		puts ARGS((const char *str));
int		putw ARGS((int integer, FILE *file_ptr));

#if    OS_VAXVMS
int		rewind ARGS((FILE *file_ptr));
					/* library declaration wrong, sigh.. */
#else  /* NOT OS_VAXVMS */
void		rewind ARGS((FILE *file_ptr));
#endif /* OS_VAXVMS */

int		scanf ARGS((const char *format_spec, ...));

#ifndef ardent
void		setbuf ARGS((FILE *file_ptr, char *buffer));
int		setvbuf ARGS((FILE *file_ptr, char *buffer, int type,
			size_t size));
#endif /* ardent */

#ifdef ultrix
char		*sprintf ARGS((char *str, const char *format_spec, ...));
#else /* NOT ultrix */
#ifdef apollo
char		*sprintf ARGS((char *str, char *format_spec, ...));
#else /* NOT apollo */
#ifdef sun
char		*sprintf ARGS((char *str, const char *format_spec, ...));
#else
int		sprintf ARGS((char *str, const char *format_spec, ...));
#endif /* sun */
#endif /* apollo */
#endif /* ultrix */

int		sscanf ARGS((const char *str, const char *format_spec, ...));
FILE		*tmpfile ARGS((void));
char		*tmpnam ARGS((char *name));
int		ungetc ARGS((int character, FILE *file_ptr));

#ifndef ardent
int		vfprintf ARGS((FILE *file_ptr, const char *format, void *arg));
int		vprintf ARGS((const char *format, void *arg));
int		vsprintf ARGS((char *s, const char *format, void *arg));
#endif /* ardent */

#endif /* STDC || HAVE_STDLIB_H */

#ifdef VMS

/***********************************************************************
Under VAX VMS 3.x  and  4.x, exit(n) and return(n)  treat n as a VAX VMS
status code, where odd n means success, and even n means failure.  Under
VMS 5.x, return(n)  still acts this way, but  exit(n) means success if n
== 0,  and failure  if n !=  0,  bringing it into  agreement  with other
exit() implementations.  Prior to VMS 5.x, EXIT_FAILURE and EXIT_SUCCESS
were not defined in stdlib.h; they  are in VMS 5.x.   The code here will
therefore leave the new definitions unchanged,  and supply them on older
VMS systems.
***********************************************************************/

#ifndef EXIT_FAILURE
#define EXIT_FAILURE	((1 << 28) + 2) /* (suppresses %NONAME-E-NOMSG) */
#endif /* EXIT_FAILURE */

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS	(1)
#endif /* EXIT_SUCCESS */

#else  /* NOT VMS */

#if    STDC
#else
#ifndef EXIT_FAILURE
#define EXIT_FAILURE	(1)
#endif /* EXIT_FAILURE */

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS	(0)
#endif /* EXIT_SUCCESS */
#endif /* STDC */

#endif /* VMS */

#endif /* STDLIB_H_DEFINED__ */
